@@ -363,6 +363,10 @@
                 #!+sb-doc
                 ,doc
                 (if numbers
+                    ;; FIXME: using NTH here produces
+                    ;; "caught STYLE-WARNING:
+                    ;;  The binding of RESULT is not a NUMBER"
+                    ;; Same warning occurs in -, /, =, /=, etc
                     (do ((result (nth 0 numbers) (,op result (nth i numbers)))
                          (i 1 (1+ i)))
                         ((>= i (length numbers))
@@ -780,9 +784,10 @@
   #!+sb-doc
   "Return T if all of its arguments are numerically equal, NIL otherwise."
   (declare (number number))
-  (dotimes (i (length more-numbers) t)
-    (unless (= number (nth i more-numbers))
-      (return nil))))
+  (do-rest-arg ((n i) more-numbers 0 t)
+    (unless (= number n)
+      (return (do-rest-arg ((n) more-numbers (1+ i))
+                (the number n)))))) ; for effect
 
 (defun /= (number &rest more-numbers)
   #!+sb-doc
@@ -793,23 +798,22 @@
             (i 0 (1+ i)))
           ((>= i (length more-numbers))
            t)
-        (do ((j i (1+ j)))
-            ((>= j (length more-numbers)))
-          (when (= n (nth j more-numbers))
+        (do-rest-arg ((n2) more-numbers i)
+          (when (= n n2)
             (return-from /= nil))))
       t))
 
 (macrolet ((def (op doc)
-             #!-sb-doc (declare (ignore doc))
+             (declare (ignorable doc))
              `(defun ,op (number &rest more-numbers)
                 #!+sb-doc ,doc
-                (let ((n number))
-                  (declare (number n))
-                  (dotimes (i (length more-numbers) t)
-                    (let ((arg (nth i more-numbers)))
-                      (if (,op n arg)
-                        (setf n arg)
-                        (return-from ,op nil))))))))
+                (let ((n1 number))
+                  (declare (real n1))
+                  (do-rest-arg ((n2 i) more-numbers 0 t)
+                    (if (,op n1 n2)
+                        (setf n1 n2)
+                        (return (do-rest-arg ((n) more-numbers (1+ i))
+                                  (the real n))))))))) ; for effect
   (def <  "Return T if its arguments are in strictly increasing order, NIL otherwise.")
   (def >  "Return T if its arguments are in strictly decreasing order, NIL otherwise.")
   (def <= "Return T if arguments are in strictly non-decreasing order, NIL otherwise.")
@@ -820,22 +824,20 @@
   "Return the greatest of its arguments; among EQUALP greatest, return
 the first."
   (let ((n number))
-    (declare (number n))
-    (dotimes (i (length more-numbers) n)
-      (let ((arg (nth i more-numbers)))
-        (when (> arg n)
-          (setf n arg))))))
+    (declare (real n))
+    (do-rest-arg ((arg) more-numbers 0 n)
+      (when (> arg n)
+        (setf n arg)))))
 
 (defun min (number &rest more-numbers)
   #!+sb-doc
   "Return the least of its arguments; among EQUALP least, return
 the first."
   (let ((n number))
-    (declare (number n))
-    (dotimes (i (length more-numbers) n)
-      (let ((arg (nth i more-numbers)))
-        (when (< arg n)
-          (setf n arg))))))
+    (declare (real n))
+    (do-rest-arg ((arg) more-numbers 0 n)
+      (when (< arg n)
+        (setf n arg)))))
 
 (eval-when (:compile-toplevel :execute)
 
