@@ -16,42 +16,42 @@
 ;;; FIXME: Many of these have nontrivial types, e.g. *PRINT-LEVEL*,
 ;;; *PRINT-LENGTH*, and *PRINT-LINES* are (OR NULL UNSIGNED-BYTE).
 
-(defvar *print-readably* nil
+(!defvar *print-readably* nil
   #!+sb-doc
   "If true, all objects will be printed readably. If readable printing
   is impossible, an error will be signalled. This overrides the value of
   *PRINT-ESCAPE*.")
-(defvar *print-escape* t
+(!defvar *print-escape* t
   #!+sb-doc
   "Should we print in a reasonably machine-readable way? (possibly
   overridden by *PRINT-READABLY*)")
-(defvar *print-pretty* nil ; (set later when pretty-printer is initialized)
+(!defvar *print-pretty* nil ; (set later when pretty-printer is initialized)
   #!+sb-doc
   "Should pretty printing be used?")
-(defvar *print-base* 10.
+(!defvar *print-base* 10.
   #!+sb-doc
   "The output base for RATIONALs (including integers).")
-(defvar *print-radix* nil
+(!defvar *print-radix* nil
   #!+sb-doc
   "Should base be verified when printing RATIONALs?")
-(defvar *print-level* nil
+(!defvar *print-level* nil
   #!+sb-doc
   "How many levels should be printed before abbreviating with \"#\"?")
-(defvar *print-length* nil
+(!defvar *print-length* nil
   #!+sb-doc
   "How many elements at any level should be printed before abbreviating
   with \"...\"?")
-(defvar *print-circle* nil
+(!defvar *print-circle* nil
   #!+sb-doc
   "Should we use #n= and #n# notation to preserve uniqueness in general (and
   circularity in particular) when printing?")
-(defvar *print-case* :upcase
+(!defvar *print-case* :upcase
   #!+sb-doc
   "What case should the printer should use default?")
-(defvar *print-array* t
+(!defvar *print-array* t
   #!+sb-doc
   "Should the contents of arrays be printed?")
-(defvar *print-gensym* t
+(!defvar *print-gensym* t
   #!+sb-doc
   "Should #: prefixes be used when printing symbols with null SYMBOL-PACKAGE?")
 (defvar *print-lines* nil
@@ -70,7 +70,7 @@
 #!+sb-doc
 (setf (fdocumentation '*print-pprint-dispatch* 'variable)
       "The pprint-dispatch-table that controls how to pretty-print objects.")
-(defvar *suppress-print-errors* nil
+(!defvar *suppress-print-errors* nil
   #!+sb-doc
   "Suppress printer errors when the condition is of the type designated by this
 variable: an unreadable object representing the error is printed instead.")
@@ -625,12 +625,9 @@ variable: an unreadable object representing the error is printed instead.")
               (let ((prefix (or (car (rassoc package (package-%local-nicknames current)))
                                 (package-name package))))
                 (output-symbol-name prefix stream))
-              (multiple-value-bind (symbol externalp)
-                  (find-external-symbol name package)
-                (declare (ignore symbol))
-                (if externalp
-                    (write-char #\: stream)
-                    (write-string "::" stream)))))))
+              (if (nth-value 1 (find-external-symbol name package))
+                  (write-char #\: stream)
+                  (write-string "::" stream))))))
         (output-symbol-name name stream))
       (output-symbol-name (symbol-name object) stream nil)))
 
@@ -641,7 +638,10 @@ variable: an unreadable object representing the error is printed instead.")
   (declare (type simple-string name))
   (let ((*readtable* (if *print-readably* *standard-readtable* *readtable*)))
     (setup-printer-state)
-    (if (and maybe-quote (symbol-quotep name))
+    (if (and maybe-quote (or
+                          (and (readtable-normalization *readtable*)
+                               (not (sb!unicode:normalized-p name :nfkc)))
+                          (symbol-quotep name)))
         (output-quoted-symbol-name name stream)
         (funcall *internal-symbol-output-fun* name stream))))
 
@@ -686,6 +686,25 @@ variable: an unreadable object representing the error is printed instead.")
 
 ) ; EVAL-WHEN
 
+;;; For each character, the value of the corresponding element is the
+;;; lowest base in which that character is a digit.
+(declaim (type (simple-array (unsigned-byte 8) (128)) ; FIXME: range?
+               *digit-bases*))
+(defvar *digit-bases*
+  (make-array 128 ; FIXME
+              :element-type '(unsigned-byte 8)))
+
+(defun !printer-cold-init ()
+(setq *digit-bases* (make-array 128 ; FIXME
+                                :element-type '(unsigned-byte 8)
+                                :initial-element 36)
+      *character-attributes* (make-array 160 ; FIXME
+                                         :element-type '(unsigned-byte 16)
+                                         :initial-element 0))
+(dotimes (i 36)
+  (let ((char (digit-char i 36)))
+    (setf (aref *digit-bases* (char-code char)) i)))
+
 (flet ((set-bit (char bit)
          (let ((code (char-code char)))
            (setf (aref *character-attributes* code)
@@ -716,18 +735,7 @@ variable: an unreadable object representing the error is printed instead.")
   (dotimes (i 160) ; FIXME
     (when (zerop (aref *character-attributes* i))
       (setf (aref *character-attributes* i) funny-attribute))))
-
-;;; For each character, the value of the corresponding element is the
-;;; lowest base in which that character is a digit.
-(defvar *digit-bases*
-  (make-array 128 ; FIXME
-              :element-type '(unsigned-byte 8)
-              :initial-element 36))
-(declaim (type (simple-array (unsigned-byte 8) (#.128)) ; FIXME
-               *digit-bases*))
-(dotimes (i 36)
-  (let ((char (digit-char i 36)))
-    (setf (aref *digit-bases* (char-code char)) i)))
+) ; end !COLD-PRINT-INIT
 
 ;;; A FSM-like thingie that determines whether a symbol is a potential
 ;;; number or has evil characters in it.
@@ -1746,7 +1754,7 @@ variable: an unreadable object representing the error is printed instead.")
                            (standard-char-p char)))
             (name (char-name char)))
         (write-string "#\\" stream)
-        (if (and name (not graphicp))
+        (if (and name (or (not graphicp) *print-readably*))
             (quote-string name stream)
             (write-char char stream)))
       (write-char char stream)))
