@@ -16,7 +16,7 @@
 ;;; ANSI: "the floating-point format that is to be used when reading a
 ;;; floating-point number that has no exponent marker or that has e or
 ;;; E for an exponent marker"
-(defvar *read-default-float-format* 'single-float)
+(!defvar *read-default-float-format* 'single-float)
 (declaim (type (member short-float single-float double-float long-float)
                *read-default-float-format*))
 
@@ -40,9 +40,6 @@
 ;;; In case we get an error trying to parse a symbol, we want to rebind the
 ;;; above stuff so it's cool.
 
-;;; FIXME: These forward declarations should be moved somewhere earlier,
-;;; or discarded.
-(declaim (special *package* *read-base*))
 
 ;;;; reader errors
 
@@ -99,9 +96,6 @@
   (declare (character char) (readtable readtable))
   (if (typep char 'base-char)
       (svref (character-macro-array readtable) (char-code char))
-      ;; Note: DEFAULT here is NIL, not #'UNDEFINED-MACRO-CHAR, so
-      ;; that everything above the base-char range is a non-macro
-      ;; constituent by default.
       (values (gethash char (character-macro-hash-table readtable) nil))))
 
 ;; As above but get the entry for SUB-CHAR in a dispatching macro table.
@@ -133,25 +127,6 @@
   `(let ((x ,val))
      (if (fdefn-p x) (fdefn-name x) x)))
 
-;;; the value represented by whatever is stored in the character macro
-;;; table. As per ANSI #'GET-MACRO-CHARACTER and #'SET-MACRO-CHARACTER,
-;;; a function value represents itself, and a NIL value represents the
-;;; default behavior.
-(defmacro !with-char-macro-result ((result-var supplied-p-var)
-                                   (stream char) &body body)
-  (with-unique-names (proc)
-    `(dx-flet ((,proc (&optional (,result-var nil ,supplied-p-var) &rest junk)
-                 (declare (ignore junk)) ; is this ANSI-specified?
-                 ,@body))
-       (multiple-value-call #',proc
-         (let ((entry (get-raw-cmt-entry ,char *readtable*)))
-           (funcall (!cmt-entry-to-function entry #'read-token)
-                    ,stream ,char))))))
-
-(defun undefined-macro-char (stream char)
-  (unless *read-suppress*
-    (simple-reader-error stream "undefined read-macro character ~S" char)))
-
 ;;; The character attribute table is a BASE-CHAR-CODE-LIMIT vector
 ;;; of (unsigned-byte 8) plus a hashtable to handle higher character codes.
 
@@ -256,6 +231,21 @@
   (assert-not-standard-readtable readtable '(setf readtable-case))
   (setf (%readtable-case readtable) case))
 
+(defun readtable-normalization (readtable)
+  #!+sb-doc
+  "Returns T if READTABLE normalizes strings to NFKC, and NIL otherwise.
+The READTABLE-NORMALIZATION of the standard readtable is T."
+  (%readtable-normalization readtable))
+
+(defun (setf readtable-normalization) (new-value readtable)
+  #!+sb-doc
+  "Sets the READTABLE-NORMALIZATION of the given READTABLE to NEW-VALUE.
+Pass T to make READTABLE normalize symbols to NFKC (the default behavior),
+and NIL to suppress normalization."
+  ;; This function does not accept a readtable designator, only a readtable.
+  (assert-not-standard-readtable readtable '(setf readtable-normalization))
+  (setf (%readtable-normalization readtable) new-value))
+
 (defun replace/eql-hash-table (to from &optional (transform #'identity))
   (maphash (lambda (k v) (setf (gethash k to) (funcall transform v))) from)
   to)
@@ -306,6 +296,8 @@
      #'copy-cmt-entry)
     (setf (readtable-case really-to-readtable)
           (readtable-case really-from-readtable))
+    (setf (readtable-normalization really-to-readtable)
+          (readtable-normalization really-from-readtable))
     really-to-readtable))
 
 (defun set-syntax-from-char (to-char from-char &optional
@@ -529,7 +521,8 @@ standard Lisp readtable when NIL."
   (escapes (make-array 10 :element-type 'fixnum :fill-pointer 0 :adjustable t)
            :type (and (vector fixnum) (not simple-array)) :read-only t)
   ;; Link to next TOKEN-BUF, to chain the *TOKEN-BUF-POOL* together.
-  (next nil :type (or null token-buf)))
+  (next nil :type (or null token-buf))
+  (only-base-chars t :type boolean))
 (declaim (freeze-type token-buf))
 
 (def!method print-object ((self token-buf) stream)
@@ -552,6 +545,7 @@ standard Lisp readtable when NIL."
   (setf (fill-pointer (token-buf-escapes buffer)) 0)
   (setf (token-buf-fill-ptr buffer) 0)
   (setf (token-buf-cursor buffer) 0)
+  (setf (token-buf-only-base-chars buffer) t)
   buffer)
 
 ;; "Output" a character into the reader's buffer.
@@ -564,6 +558,8 @@ standard Lisp readtable when NIL."
     ;; an out-of-line call for the uncommon case avoids bloat.
     ;; Size should be doubled.
       (grow-read-buffer))
+    (unless (typep char 'base-char)
+      (setf (token-buf-only-base-chars buffer) nil))
     (setf (elt (token-buf-string buffer) op) char)
     (setf (token-buf-fill-ptr buffer) (1+ op))))
 
@@ -672,7 +668,7 @@ standard Lisp readtable when NIL."
 ;;; as an alist, so maybe we should. -- WHN 19991202
 (defvar *sharp-equal-alist* ())
 
-(declaim (special *standard-input*))
+(declaim (ftype (sfunction (t t) (values t t)) read-maybe-nothing))
 
 ;;; Like READ-PRESERVING-WHITESPACE, but doesn't check the read buffer
 ;;; for being set up properly.
@@ -685,7 +681,8 @@ standard Lisp readtable when NIL."
          (cond ((eq char +EOF+) (return eof-value))
                ((whitespace[2]p char))
                (t
-                (!with-char-macro-result (result result-p) (stream char)
+                (multiple-value-bind (result-p result)
+                    (read-maybe-nothing stream char)
                   ;; Repeat if macro returned nothing.
                   (when result-p
                     (return (unless *read-suppress* result))))))))
@@ -706,13 +703,18 @@ standard Lisp readtable when NIL."
   (check-for-recursive-read stream recursive-p 'read-preserving-whitespace)
   (%read-preserving-whitespace stream eof-error-p eof-value recursive-p))
 
-;;; Return NIL or a list with one thing, depending.
-;;;
+;;; Read from STREAM given starting CHAR, returning T and the resulting
+;;; object, unless CHAR is a macro yielding no value, then NIL and NIL,
 ;;; for functions that want comments to return so that they can look
 ;;; past them. We assume CHAR is not whitespace.
 (defun read-maybe-nothing (stream char)
-  (!with-char-macro-result (retval retval-p) (stream char)
-    (if retval-p (list retval))))
+  (multiple-value-call
+   (lambda (&optional (result nil supplied-p) &rest junk)
+     (declare (ignore junk)) ; is this ANSI-specified?
+     (values supplied-p result))
+   (funcall (!cmt-entry-to-function
+             (get-raw-cmt-entry char *readtable*) #'read-token)
+            stream char)))
 
 (defun read (&optional (stream *standard-input*)
                        (eof-error-p t)
@@ -747,8 +749,10 @@ standard Lisp readtable when NIL."
                 (retlist ()))
                ((char= char endchar)
                 (unless *read-suppress* (nreverse retlist)))
-             (setq retlist (nconc (read-maybe-nothing input-stream char)
-                                  retlist)))))
+             (multiple-value-bind (winp obj)
+                 (read-maybe-nothing input-stream char)
+               (when winp
+                 (push obj retlist))))))
     (declare (inline %read-delimited-list))
     (if recursive-p
         (%read-delimited-list endchar input-stream)
@@ -804,9 +808,7 @@ standard Lisp readtable when NIL."
                                "Nothing appears before . in list.")))
                        ((whitespace[2]p nextchar)
                         (setq nextchar (flush-whitespace stream))))
-                 (rplacd listtail
-                             ;; Return list containing last thing.
-                         (car (read-after-dot stream nextchar)))
+                 (rplacd listtail (read-after-dot stream nextchar))
                  ;; Check for improper ". ,@" or ". ,." now rather than
                  ;; in the #\` reader. The resulting QUASIQUOTE macro might
                  ;; never be exapanded, but nonetheless could be erroneous.
@@ -820,11 +822,10 @@ standard Lisp readtable when NIL."
                     ;; Put back NEXTCHAR so that we can read it normally.
                 (t (unread-char nextchar stream)))))
       ;; Next thing is not an isolated dot.
-      (let ((listobj (read-maybe-nothing stream firstchar)))
+      (multiple-value-bind (winp obj) (read-maybe-nothing stream firstchar)
         ;; allows the possibility that a comment was read
-        (when listobj
-          (rplacd listtail listobj)
-          (setq listtail listobj))))))
+        (when winp
+          (setq listtail (cdr (rplacd listtail (list obj)))))))))
 
 (defun read-after-dot (stream firstchar)
   ;; FIRSTCHAR is non-whitespace!
@@ -835,18 +836,17 @@ standard Lisp readtable when NIL."
              (return-from read-after-dot nil)
              (simple-reader-error stream "Nothing appears after . in list.")))
       ;; See whether there's something there.
-      (setq lastobj (read-maybe-nothing stream char))
-      (when lastobj (return t)))
+      (multiple-value-bind (winp obj) (read-maybe-nothing stream char)
+        (when winp (return (setq lastobj obj)))))
     ;; At least one thing appears after the dot.
     ;; Check for more than one thing following dot.
-    (do ((lastchar (flush-whitespace stream)
-                   (flush-whitespace stream)))
-        ((char= lastchar #\) ) lastobj) ;success!
-      ;; Try reading virtual whitespace.
-      (if (and (read-maybe-nothing stream lastchar)
-               (not *read-suppress*))
-          (simple-reader-error stream
-                               "More than one object follows . in list.")))))
+    (loop
+     (let ((char (flush-whitespace stream)))
+       (cond ((char= #\) char) (return lastobj)) ;success!
+             ;; Try reading virtual whitespace.
+             ((and (read-maybe-nothing stream char) (not *read-suppress*))
+              (simple-reader-error
+               stream "More than one object follows . in list.")))))))
 
 (defun read-string (stream closech)
   ;; This accumulates chars until it sees same char that invoked it.
@@ -882,6 +882,7 @@ standard Lisp readtable when NIL."
 ;;; -- a TOKEN-BUF
 ;;; -- whether any escape character was seen (even if no character is escaped)
 ;;; -- whether a package delimiter character was seen
+;;; Normalizes the input to NFKC before returning
 (defun internal-read-extended-token (stream firstchar escape-firstchar
                                      &aux (read-buffer *read-buffer*))
   (reset-read-buffer read-buffer)
@@ -897,10 +898,13 @@ standard Lisp readtable when NIL."
               (unread-char char stream)
               t)
              (t nil))
-       (values read-buffer
-               (or (plusp (fill-pointer (token-buf-escapes read-buffer)))
-                   seen-multiple-escapes)
-               colon))
+       (progn
+         (multiple-value-setq (read-buffer colon)
+           (normalize-read-buffer read-buffer colon))
+         (values read-buffer
+                 (or (plusp (fill-pointer (token-buf-escapes read-buffer)))
+                     seen-multiple-escapes)
+                 colon)))
     (cond ((single-escape-p char rt)
            ;; It can't be a number, even if it's 1\23.
            ;; Read next char here, so it won't be casified.
@@ -1012,6 +1016,43 @@ standard Lisp readtable when NIL."
   "the radix that Lisp reads numbers in")
 (declaim (type (integer 2 36) *read-base*))
 
+;;; Normalize TOKEN-BUF to NFKC, returning a new TOKEN-BUF and the
+;;; COLON value
+(defun normalize-read-buffer (token-buf &optional colon)
+  (unless (readtable-normalization *readtable*)
+    (return-from normalize-read-buffer (values token-buf colon)))
+  (when (token-buf-only-base-chars token-buf)
+    (return-from normalize-read-buffer (values token-buf colon)))
+  (let ((current-buffer (copy-token-buf-string token-buf))
+        (old-escapes (copy-seq (token-buf-escapes token-buf)))
+        (str-to-normalize (make-string (token-buf-fill-ptr token-buf)))
+        (normalize-ptr 0) (escapes-ptr 0))
+    (reset-read-buffer token-buf)
+    (macrolet ((clear-str-to-normalize ()
+               `(progn
+                  (loop for char across (sb!unicode:normalize-string
+                                         (subseq str-to-normalize 0 normalize-ptr)
+                                         :nfkc) do
+                       (ouch-read-buffer char token-buf))
+                  (setf normalize-ptr 0)))
+               (push-to-normalize (ch)
+                 (let ((ch-gen (gensym)))
+                   `(let ((,ch-gen ,ch))
+                      (setf (char str-to-normalize normalize-ptr) ,ch-gen)
+                      (incf normalize-ptr)))))
+      (loop for c across current-buffer
+         for i from 0
+         do
+           (if (and (< escapes-ptr (length old-escapes))
+                    (eql i (aref old-escapes escapes-ptr)))
+               (progn
+                 (clear-str-to-normalize)
+                 (ouch-read-buffer-escaped c token-buf)
+                 (incf escapes-ptr))
+               (push-to-normalize c)))
+      (clear-str-to-normalize)
+      (values token-buf colon))))
+
 ;;; Modify the read buffer according to READTABLE-CASE, ignoring
 ;;; ESCAPES. ESCAPES is a vector of the escaped indices.
 (defun casify-read-buffer (token-buf)
@@ -1377,6 +1418,7 @@ extended <package-name>::<form-in-package> syntax."
       (unless (zerop colons)
         (simple-reader-error
          stream "too many colons in ~S" (copy-token-buf-string buf)))
+      (setf buf (normalize-read-buffer buf))
       (casify-read-buffer buf)
       (setq colons 1)
       (setq package-designator
@@ -1420,6 +1462,7 @@ extended <package-name>::<form-in-package> syntax."
                               package-designator))
         (t (go SYMBOL)))
       RETURN-SYMBOL
+      (setf buf (normalize-read-buffer buf))
       (casify-read-buffer buf)
       (let ((pkg (if package-designator
                      (reader-find-package package-designator stream)
