@@ -9,6 +9,14 @@
 
 (in-package "SB-COLD")
 
+(export '*symbol-values-for-genesis*)
+(let ((pathname "output/init-symbol-values.lisp-expr"))
+  (defvar *symbol-values-for-genesis*
+    (and (probe-file pathname) (read-from-file pathname)))
+  (defun save-initial-symbol-values ()
+    (with-open-file (f pathname :direction :output :if-exists :supersede)
+      (write *symbol-values-for-genesis* :stream f :readably t))))
+
 ;;; Either load or compile-then-load the cross-compiler into the
 ;;; cross-compilation host Common Lisp.
 (defun load-or-cload-xcompiler (load-or-cload-stem)
@@ -115,6 +123,7 @@
                     "DEPOSIT-FIELD" "DPB"
                     "FBOUNDP" "FDEFINITION" "FMAKUNBOUND"
                     "FIND-CLASS"
+                    "GENSYM" "*GENSYM-COUNTER*"
                     "GET-SETF-EXPANSION"
                     "LDB" "LDB-TEST"
                     "LISP-IMPLEMENTATION-TYPE" "LISP-IMPLEMENTATION-VERSION"
@@ -161,9 +170,7 @@
   ;; routines.
   (do-stems-and-flags (stem flags)
     (unless (find :not-host flags)
-      (funcall load-or-cload-stem
-               stem
-               :ignore-failure-p (find :ignore-failure-p flags))
+      (funcall load-or-cload-stem stem flags)
       #!+sb-show (warn-when-cl-snapshot-diff *cl-snapshot*)))
 
   ;; If the cross-compilation host is SBCL itself, we can use the
