@@ -50,7 +50,7 @@
 
 ;;; *CONSTRAINT-UNIVERSE* gets bound in IR1-PHASES to a fresh,
 ;;; zero-length, non-zero-total-size vector-with-fill-pointer.
-(declaim (type (and vector (not simple-vector)) *constraint-universe*))
+(declaim (type (and (vector t) (not simple-array)) *constraint-universe*))
 (defvar *constraint-universe*)
 
 (deftype constraint-y () '(or ctype lvar lambda-var constant))
@@ -1036,18 +1036,22 @@
           (let* ((node (block-last block))
                  (old-consequent-constraints (if-consequent-constraints node))
                  (old-alternative-constraints (if-alternative-constraints node))
+                 (no-consequent (conset-empty consequent-constraints))
+                 (no-alternative (conset-empty alternative-constraints))
                  (succ ()))
             ;; Add the consequent and alternative constraints to GEN.
-            (cond ((conset-empty consequent-constraints)
+            (cond ((and no-consequent no-alternative)
                    (setf (if-consequent-constraints node) gen)
                    (setf (if-alternative-constraints node) gen))
                   (t
                    (setf (if-consequent-constraints node) (copy-conset gen))
-                   (conset-union (if-consequent-constraints node)
-                                 consequent-constraints)
+                   (unless no-consequent
+                     (conset-union (if-consequent-constraints node)
+                                   consequent-constraints))
                    (setf (if-alternative-constraints node) gen)
-                   (conset-union (if-alternative-constraints node)
-                                 alternative-constraints)))
+                   (unless no-alternative
+                     (conset-union (if-alternative-constraints node)
+                                   alternative-constraints))))
             ;; Has the consequent been changed?
             (unless (and old-consequent-constraints
                          (conset= (if-consequent-constraints node)
