@@ -47,7 +47,19 @@
 ;;; the type of LAYOUT-DEPTHOID slot values
 (def!type layout-depthoid () '(or index (integer -1 -1)))
 
-;;; possible values for the INLINE-ness of a function.
+;;; An INLINEP value describes how a function is called. The values
+;;; have these meanings:
+;;;     NIL     No declaration seen: do whatever you feel like, but don't
+;;;             dump an inline expansion.
+;;; :NOTINLINE  NOTINLINE declaration seen: always do full function call.
+;;;    :INLINE  INLINE declaration seen: save expansion, expanding to it
+;;;             if policy favors.
+;;; :MAYBE-INLINE
+;;;             Retain expansion, but only use it opportunistically.
+;;;             :MAYBE-INLINE is quite different from :INLINE. As explained
+;;;             by APD on #lisp 2005-11-26: "MAYBE-INLINE lambda is
+;;;             instantiated once per component, INLINE - for all
+;;;             references (even under #'without FUNCALL)."
 (deftype inlinep ()
   '(member :inline :maybe-inline :notinline nil))
 (defparameter *inlinep-translations*
@@ -97,6 +109,7 @@
 (defvar *handled-conditions*)
 (defvar *disabled-package-locks*)
 (defvar *policy*)
+(defvar *macro-policy* nil)
 (defvar *dynamic-counts-tn*)
 (defvar *elsewhere*)
 (defvar *event-info*)
