@@ -218,6 +218,9 @@
 (defknown (use-package unuse-package)
   ((or list package-designator) &optional package-designator) (eql t))
 (defknown find-all-symbols (string-designator) list (flushable))
+;; private
+(defknown package-iter-step (fixnum index simple-vector list)
+  (values fixnum index simple-vector list symbol symbol))
 
 ;;;; from the "Numbers" chapter:
 
@@ -402,11 +405,16 @@
 (defknown random ((or (float (0.0)) (integer 1)) &optional random-state)
   (or (float 0.0) (integer 0))
   (explicit-check))
-(defknown make-random-state (&optional
+(defknown make-random-state (&optional (or random-state (member nil t)))
+  random-state (flushable))
+;; The arg type specifier is for documentation only- EXPLICIT-CHECK prevails.
+;; It feels almost wrong to state it at all.
+(defknown seed-random-state (&optional ; SBCL extension
                              (or (member nil t) random-state unsigned-byte
                                  (simple-array (unsigned-byte 8) (*))
                                  (simple-array (unsigned-byte 32) (*))))
-  random-state (flushable))
+  random-state (flushable explicit-check))
+
 (defknown random-state-p (t) boolean (movable foldable flushable))
 
 ;;;; from the "Characters" chapter:
@@ -417,12 +425,21 @@
 (defknown digit-char-p (character &optional (integer 2 36))
   (or (integer 0 35) null) (movable foldable flushable))
 
+;; Character predicates: if the 2-argument predicate which underlies
+;; the N-argument predicate unavoidably type-checks its 2 args,
+;; then the N-argument form should not type-check anything
+;; except in the degenerate case of 1 actual argument.
+;; All of the case-sensitive functions have the check of the first arg
+;; generated by the compiler, and successive args checked by hand.
+;; The case-insensitive functions don't need any checks, since the underlying
+;; two-arg case-insensitive function does it, except when it isn't called.
 (defknown (char=)
   (character &rest character) boolean (movable foldable flushable commutative))
 
-(defknown (char/= char< char> char<= char>= char-equal char-not-equal
-                 char-lessp char-greaterp char-not-greaterp char-not-lessp)
+(defknown (char/= char< char> char<= char>= char-not-equal)
   (character &rest character) boolean (movable foldable flushable))
+(defknown (char-equal char-lessp char-greaterp char-not-greaterp char-not-lessp)
+  (character &rest character) boolean (movable foldable flushable explicit-check))
 
 (defknown (two-arg-char-equal)
     (character character) boolean (movable foldable flushable commutative))
@@ -632,13 +649,15 @@
                     (:test-not callable) (:start index) (:from-end t)
                     (:end sequence-end) (:key callable))
   (or index null)
-  (foldable flushable call))
+  (foldable flushable call)
+  :derive-type #'position-derive-type)
 
 (defknown (position-if position-if-not)
   (callable sequence &rest t &key (:from-end t) (:start index)
    (:end sequence-end) (:key callable))
   (or index null)
-  (foldable flushable call))
+  (foldable flushable call)
+  :derive-type #'position-derive-type)
 
 (defknown count (t sequence &rest t &key
                    (:test callable) (:test-not callable) (:start index)
@@ -1200,9 +1219,11 @@
 ;;; xxx-TO-STRING functions are not foldable because they depend on
 ;;; the dynamic environment, the state of the pretty printer dispatch
 ;;; table, and probably other run-time factors.
-  (deffrob write-to-string () simple-string (flushable explicit-check)))
+  (deffrob write-to-string () simple-string
+           (unsafely-flushable explicit-check)))
 
 (defknown (prin1-to-string princ-to-string) (t) simple-string (flushable))
+(defknown sb!impl::stringify-object (t) simple-string)
 
 (defknown write-char (character &optional stream-designator) character
   (explicit-check)
