@@ -732,34 +732,42 @@
           (values (sb!xc:compiler-macro-function opname *lexenv*) opname)
           (values nil nil))))
 
-;;; Picks of special forms and compiler-macro expansions, and hands
+;;; If FORM has a usable compiler macro, use it; otherwise return FORM itself.
+;;; Return the name of the compiler-macro as a secondary value, if applicable.
+(defun expand-compiler-macro (form)
+  (multiple-value-bind (cmacro-fun cmacro-fun-name)
+      (find-compiler-macro (car form) form)
+    (if (and cmacro-fun
+             ;; CLHS 3.2.2.1.3 specifies that NOTINLINE
+             ;; suppresses compiler-macros.
+             (not (fun-lexically-notinline-p cmacro-fun-name)))
+        (values (handler-case (careful-expand-macro cmacro-fun form t)
+                  (compiler-macro-keyword-problem (c)
+                    (print-compiler-message *error-output* "note: ~A" (list c))
+                    form))
+                cmacro-fun-name)
+        (values form nil))))
+
+;;; Picks off special forms and compiler-macro expansions, and hands
 ;;; the rest to IR1-CONVERT-COMMON-FUNCTOID
 (defun ir1-convert-functoid (start next result form)
   (let* ((op (car form))
          (translator (and (symbolp op) (info :function :ir1-convert op))))
     (cond (translator
+           ;; FIXME: redundant? A macro can not be defined in the first place.
            (when (sb!xc:compiler-macro-function op *lexenv*)
              (compiler-warn "ignoring compiler macro for special form"))
            (funcall translator start next result form))
           (t
-           (multiple-value-bind (cmacro-fun cmacro-fun-name)
-               (find-compiler-macro op form)
-             (if (and cmacro-fun
-                      ;; CLHS 3.2.2.1.3 specifies that NOTINLINE
-                      ;; suppresses compiler-macros.
-                      (not (fun-lexically-notinline-p cmacro-fun-name)))
-                 (let ((res (handler-case
-                                (careful-expand-macro cmacro-fun form t)
-                              (compiler-macro-keyword-problem (c)
-                                (print-compiler-message *error-output* "note: ~A" (list c))
-                                form))))
-                   (cond ((eq res form)
-                          (ir1-convert-common-functoid start next result form op))
-                         (t
-                          (unless (policy *lexenv* (zerop store-xref-data))
-                            (record-call cmacro-fun-name (ctran-block start) *current-path*))
-                          (ir1-convert start next result res))))
-                 (ir1-convert-common-functoid start next result form op)))))))
+           (multiple-value-bind (res cmacro-fun-name)
+               (expand-compiler-macro form)
+             (cond ((eq res form)
+                    (ir1-convert-common-functoid start next result form op))
+                   (t
+                    (unless (policy *lexenv* (zerop store-xref-data))
+                      (record-call cmacro-fun-name (ctran-block start)
+                                   *current-path*))
+                    (ir1-convert start next result res))))))))
 
 ;;; Handles the "common" cases: any other forms except special forms
 ;;; and compiler-macros.
@@ -1496,6 +1504,7 @@
   (declare (type list raw-spec vars fvars))
   (declare (type lexenv res))
   (let ((spec (canonized-decl-spec raw-spec))
+        (optimize-qualities)
         (result-type *wild-type*))
     (values
      (case (first spec)
@@ -1510,9 +1519,10 @@
         (process-ignore-decl spec vars fvars)
         res)
        (optimize
-        (make-lexenv
-         :default res
-         :policy (process-optimize-decl spec (lexenv-policy res))))
+        (multiple-value-bind (new-policy specified-qualities)
+            (process-optimize-decl spec (lexenv-policy res))
+          (setq optimize-qualities specified-qualities)
+          (make-lexenv :default res :policy new-policy)))
        (muffle-conditions
         (make-lexenv
          :default res
@@ -1549,7 +1559,8 @@
           (if fn
               (funcall fn res spec vars fvars)
               res))))
-     result-type)))
+     result-type
+     optimize-qualities)))
 
 ;;; Use a list of DECLARE forms to annotate the lists of LAMBDA-VAR
 ;;; and FUNCTIONAL structures which are being bound. In addition to
@@ -1566,23 +1577,31 @@
                       (lexenv *lexenv*) (binding-form-p nil) (context :compile))
   (declare (list decls vars fvars))
   (let ((result-type *wild-type*)
+        (optimize-qualities)
         (*post-binding-variable-lexenv* nil))
     (dolist (decl decls)
       (dolist (spec (rest decl))
-        (progv
+        (flet
+            ((process-it ()
+               (unless (consp spec)
+                 (compiler-error "malformed declaration specifier ~S in ~S"
+                                 spec decl))
+               (multiple-value-bind (new-env new-result-type new-qualities)
+                   (process-1-decl spec lexenv vars fvars binding-form-p context)
+                 (setq lexenv new-env
+                       optimize-qualities (nconc new-qualities optimize-qualities))
+                 (unless (eq new-result-type *wild-type*)
+                   (setq result-type
+                         (values-type-intersection result-type new-result-type))))))
+          (if (eq context :compile)
+              (let ((*current-path* (or (get-source-path spec)
+                                        (get-source-path decl)
+                                        *current-path*)))
+                (process-it))
             ;; Kludge: EVAL calls this function to deal with LOCALLY.
-            (when (eq context :compile) (list '*current-path*))
-            (when (eq context :compile) (list (or (get-source-path spec)
-                                                  (get-source-path decl)
-                                                  *current-path*)))
-          (unless (consp spec)
-            (compiler-error "malformed declaration specifier ~S in ~S" spec decl))
-          (multiple-value-bind (new-env new-result-type)
-              (process-1-decl spec lexenv vars fvars binding-form-p context)
-            (setq lexenv new-env)
-            (unless (eq new-result-type *wild-type*)
-              (setq result-type
-                    (values-type-intersection result-type new-result-type)))))))
+              (process-it)))))
+    (advise-if-repeated-optimize-qualities
+     (lexenv-policy lexenv) optimize-qualities)
     (values lexenv result-type *post-binding-variable-lexenv*)))
 
 (defun %processing-decls (decls vars fvars ctran lvar binding-form-p fun)
