@@ -814,6 +814,7 @@
   (def string=* not)
   (def string/=* identity))
 
+(deftransform string ((x) (symbol)) '(symbol-name x))
 
 ;;;; transforms for sequence functions
 
@@ -1222,7 +1223,7 @@
                              (member string simple-string base-string simple-base-string))
                             &rest sequence)
                            * :node node)
-  (let ((vars (loop for x in lvars collect (gensym)))
+  (let ((vars (loop for x in lvars collect (sb!xc:gensym)))
         (type (lvar-value result-type)))
     (if (policy node (<= speed space))
         ;; Out-of-line
@@ -1263,8 +1264,9 @@
                      (.pos. ,non-constant-start)
                      (.string. (make-string .length. :element-type ',element-type)))
                 (declare (type index .length. .pos.)
-                         (muffle-conditions compiler-note))
-                ,@(loop with first-constants = t
+                         (muffle-conditions compiler-note)
+                         (ignorable .pos.))
+                ,@(loop with constants = -1
                         for first = t then nil
                         for value in lvar-values
                         for var in vars
@@ -1279,11 +1281,11 @@
                                           ;; Without truly-the we get massive numbers
                                           ;; of pointless error traps.
                                           `(setf (aref .string.
-                                                       (truly-the index ,(if first-constants
-                                                                             i
+                                                       (truly-the index ,(if constants
+                                                                             (incf constants)
                                                                              `(+ .pos. ,i))))
                                                  ,c))
-                                  ,(unless first-constants
+                                  ,(unless constants
                                      `(incf (truly-the index .pos.) ,(length value)))))
                               (t
                                (prog1
@@ -1294,12 +1296,12 @@
                                          t)
                                         ,var
                                       (replace .string. ,var
-                                               ,@(cond ((not first-constants)
+                                               ,@(cond ((not constants)
                                                         '(:start1 .pos.))
                                                        ((plusp non-constant-start)
                                                         `(:start1 ,non-constant-start))))
                                       (incf (truly-the index .pos.) (length ,var)))
-                                 (setf first-constants nil)))))
+                                 (setf constants nil)))))
                 .string.))
             lvars)))))
 
