@@ -25,11 +25,25 @@
 #include "genesis/code.h"
 
 void gc_free_heap(void);
-inline page_index_t find_page_index(void *);
-inline void *page_address(page_index_t);
+extern page_index_t find_page_index(void *);
+extern void *page_address(page_index_t);
 int gencgc_handle_wp_violation(void *);
 
 
+#if GENCGC_CARD_BYTES > USHRT_MAX
+# if GENCGC_CARD_BYTES > UINT_MAX
+#   error "GENCGC_CARD_BYTES unexpectedly large."
+# else
+#   define PAGE_BYTES_FMT "u"
+    typedef unsigned int page_bytes_t;
+# endif
+#else
+# define PAGE_BYTES_FMT "hu"
+  typedef unsigned short page_bytes_t;
+#endif
+
+typedef boolean in_use_marker_t;
+
 /* Note that this structure is also used from Lisp-side in
  * src/code/room.lisp, and the Lisp-side structure layout is currently
  * not groveled from C code but hardcoded. Any changes to the
@@ -39,24 +53,23 @@ int gencgc_handle_wp_violation(void *);
  * output the C version in genesis. -- JES, 2006-12-30.
  */
 struct page {
-    /* The name of this field is not well-chosen for its actual use.
-     * This is the offset from the start of the page to the start
-     * of the alloc_region which contains/contained it.  It's negative or 0
+    /* This is the offset from the first byte of some object in memory
+     * prior to and no closer than the start of the page to the start
+     * of the page.  Lower values here are better, 0 is ideal.  This
+     * is useful for determining where to start when scanning forward
+     * through a heap page (either for conservative root validation or
+     * for scavenging).
      */
-    long  first_object_offset;
+    os_vm_size_t scan_start_offset;
 
     /* the number of bytes of this page that are used. This may be less
      * than the actual bytes used for pages within the current
      * allocation regions. It should be 0 for all unallocated pages (not
      * hard to achieve).
      */
-#if PAGE_BYTES > USHRT_MAX
-    unsigned int bytes_used;
-#else
-    unsigned short bytes_used;
-#endif
+    page_bytes_t bytes_used;
 
-    unsigned
+    unsigned char
         /* This is set when the page is write-protected. This should
          * always reflect the actual write_protect status of a page.
          * (If the page is written into, we catch the exception, make
@@ -67,10 +80,17 @@ struct page {
          * OSes). This is useful for re-scavenging pages that are
          * written during a GC. */
         write_protected_cleared :1,
-        /* the region the page is allocated to: 0 for a free page; 1
-         * for boxed objects; 2 for unboxed objects. If the page is
-         * free the following slots are invalid (well the bytes_used
-         * must be 0). */
+        /*  000 free
+         *  ?01 boxed data
+         *  ?10 unboxed data
+         *  ?11 code
+         *  1?? open region
+         *
+         * Constants for this field are defined in gc-internal.h, the
+         * xxx_PAGE_FLAG definitions.
+         *
+         * If the page is free the following slots are invalid, except
+         * for the bytes_used which must be zero. */
         allocated :3,
         /* If this page should not be moved during a GC then this flag
          * is set. It's only valid during a GC for allocated pages. */
@@ -79,7 +99,7 @@ struct page {
          * set. No other objects should be allocated to these pages.
          * This is only valid when the page is allocated. */
         large_object :1,
-        /* True if the page is known to contain only zeroes. */
+        /* Cleared if the page is known to contain only zeroes. */
         need_to_zero :1;
 
     /* the generation that this page belongs to. This should be valid
@@ -87,6 +107,7 @@ struct page {
      * allocation region pages - this allows the space of an object to
      * be easily determined. */
     generation_index_t gen;
+    in_use_marker_t *dontmove_dwords;
 };
 
 
@@ -98,39 +119,43 @@ extern struct page *page_table;
 
 
 /* forward declarations */
-
-void sniff_code_object(struct code *code, unsigned long displacement);
+#ifdef LISP_FEATURE_X86
+void sniff_code_object(struct code *code, os_vm_size_t displacement);
 void gencgc_apply_code_fixups(struct code *old_code, struct code *new_code);
+#endif
 
-long update_dynamic_space_free_pointer(void);
-void gc_alloc_update_page_tables(int unboxed,
-                                 struct alloc_region *alloc_region);
+sword_t update_dynamic_space_free_pointer(void);
+void gc_alloc_update_page_tables(int page_type_flag, struct alloc_region *alloc_region);
 void gc_alloc_update_all_page_tables(void);
 void gc_set_region_empty(struct alloc_region *region);
 
 /*
  * predicates
  */
-static inline boolean
-space_matches_p(lispobj obj, generation_index_t space)
-{
-    page_index_t page_index=(void*)obj - (void *)DYNAMIC_SPACE_START;
-    return ((page_index >= 0)
-            && ((page_index =
-                 ((unsigned long)page_index)/PAGE_BYTES) < page_table_pages)
-            && (page_table[page_index].gen == space));
-}
 
 static inline boolean
-from_space_p(lispobj obj)
+space_matches_p(lispobj obj, generation_index_t space,
+                page_index_t *store_page_index_here)
 {
-    return space_matches_p(obj,from_space);
+    if (obj >= DYNAMIC_SPACE_START) {
+        page_index_t page_index=((pointer_sized_uint_t)obj
+                                 - DYNAMIC_SPACE_START) / GENCGC_CARD_BYTES;
+        if (store_page_index_here) {
+            *store_page_index_here = page_index;
+        }
+        return ((page_index < page_table_pages) &&
+                (page_table[page_index].gen == space));
+    } else {
+        return 0;
+    }
 }
 
+boolean from_space_p(lispobj obj);
+
 static inline boolean
 new_space_p(lispobj obj)
 {
-    return space_matches_p(obj,new_space);
+    return space_matches_p(obj, new_space, NULL);
 }
 
 extern page_index_t last_free_page;
