@@ -2514,4 +2514,82 @@
     (assert (string= (funcall f)
                      "Weird transform answer is 14"))))
 
+(defun skip-1-passthrough (a b sb-int:&more context count)
+  (declare (ignore a b))
+  (multiple-value-call 'list
+    'start
+    (sb-c::%more-arg-values context 1 (1- (truly-the fixnum count)))
+    'end))
+(defun skip-2-passthrough (a b sb-int:&more context count)
+  (declare (ignore a b))
+  (multiple-value-call 'list
+    'start
+    (sb-c::%more-arg-values context 2 (- (truly-the fixnum count) 2))
+    'end))
+(defun skip-n-passthrough (n-skip n-copy sb-int:&more context count)
+  (assert (>= count (+ n-copy n-skip))) ; prevent crashes
+  (multiple-value-call 'list
+    'start
+    (sb-c::%more-arg-values context n-skip n-copy)
+    'end))
+
+;; %MORE-ARG-VALUES was wrong on x86 and x86-64 with nonzero 'skip'.
+;; It's entirely possible that other backends are also not working.
+(test-util:with-test (:name more-arg-fancy)
+  (assert (equal (skip-1-passthrough 0 0 'a 'b 'c 'd 'e 'f)
+                 '(start b c d e f end)))
+  (assert (equal (skip-2-passthrough 0 0 'a 'b 'c 'd 'e 'f)
+                 '(start c d e f end)))
+  (assert (equal (skip-n-passthrough 1 5 'a 'b 'c 'd 'e 'f)
+                 '(start b c d e f end)))
+  (assert (equal (skip-n-passthrough 1 5 'a 'b 'c 'd 'e 'f 'g)
+                 '(start b c d e f end)))
+  (assert (equal (skip-n-passthrough 2 5 'a 'b 'c 'd 'e 'f 'g)
+                 '(start c d e f g end)))
+  (assert (equal (skip-n-passthrough 2 5 'a 'b 'c 'd 'e 'f 'g 'h)
+                 '(start c d e f g end))))
+
+(test-util:with-test (:name :macro-policy)
+  (flet ((count-notes ()
+          (let ((count 0))
+            (handler-bind ((compiler-note
+                            (lambda (c)
+                              c
+                              (incf count)
+                              (muffle-warning))))
+              (multiple-value-bind (fasl warnings errors)
+                  (compile-file "macro-policy-test.lisp"
+                               :print nil :verbose nil)
+                (ignore-errors (delete-file fasl))
+                (assert (and (not warnings) (not errors)))
+                count)))))
+    (let* ((baseline (count-notes))
+           (test (progv '(*frob-macro-policy*) '(t) (count-notes)))
+           (baseline-again (count-notes)))
+      (assert (/= 0 baseline))
+      (assert (= 0 test))
+      ;; macro-policy is rebound inside compile-file
+      (assert (= baseline-again baseline)))))
+
+(test-util:with-test (:name :redef-macro-same-file)
+  (let* ((lisp "compiler-impure-tmp.lisp")
+         (fasl (compile-file-pathname lisp)))
+    (unwind-protect
+         (let ((redef-count 0))
+           (with-open-file (f lisp :direction :output)
+             (dolist (form '((defmacro glork (x) `(car ,x))
+                             (define-compiler-macro glorpy (x) `(+ ,x 1))
+                             (defmacro glork (x) `(first ,x))
+                             (define-compiler-macro glorpy (x) `(+ ,x 2))))
+               (print form f)))
+           (multiple-value-bind (fasl warn fail)
+               (handler-bind ((sb-int:same-file-redefinition-warning
+                               (lambda (c) c (incf redef-count))))
+                 (let ((*error-output* (make-broadcast-stream)))
+                   (compile-file lisp :print nil :verbose nil)))
+             (declare (ignore fasl))
+             (assert (and warn (not fail) (= redef-count 2)))))
+      (ignore-errors (delete-file lisp))
+      (ignore-errors (delete-file fasl)))))
+
 ;;; success
